#!/usr/bin/perl
use strict;
use warnings;

use File::Spec;

#options
my $max_search_depth = 25;
my $search_path = "";

#check for arguments
if (!scalar(@ARGV) || scalar(@ARGV) % 2 != 1)
{
    print_useage();

    exit(1);
}

#parse options and arguments
my $i;

for ($i = 0; $i < scalar(@ARGV) - 1; $i += 2)
{
    my $argument = $ARGV[$i];
    my $value = $ARGV[$i + 1];

    if ($argument eq "-m")
    {
        if ($value < 0)
        {
            print STDERR "Depth search invalid, must greater than or equal to zero.\n";

            exit(3);
        }

        $max_search_depth = $value;
    }
    else
    {
        print STDERR "Unknown option ($argument).\n\n";

        print_useage();

        exit(2);
    }
}

my $argument = $ARGV[$i];

if (-f $argument)
{
    process_file($argument);
}
elsif (-d $argument)
{
    search_files_in_path($argument);
}
else
{
    print STDERR "Error: Argument " . ($i + 1) . " ($ARGV[$i]) must be file path.\n";

    exit(1);
}

#helper subroutines
sub print_useage
{
    print STDERR "Useage: [options] [search_path|file]\n";
    print STDERR "Options:\n";
    print STDERR "-m N [Max search depth, N is a number >= 0]\n";
}

sub search_files_in_path
{
    my $path = shift;

    # I'm using the closure to encapsulate data only needed by the search function, because I want to,
    # but I also want a recursive implementation; you can have your cake and eat it to.
    my $f = _search_files_in_path();

    $f->($f, $path);
}

sub _search_files_in_path
{
    my $map = {};
    my $depth = 0;

    return sub {
        my ($func, $path) = @_;

        return unless ($depth <= $max_search_depth);

        $depth++;

        opendir(my $dir, $path) or return;

        while (my $file = readdir($dir))
        {
            my $full_path = File::Spec->catfile($path, $file);

            if (-f $full_path)
            {
                process_file($full_path);
            }
            elsif (-d $full_path)
            {
                if ($file ne "." and $file ne "..")
                {
                    my $node = (stat($full_path))[1];

                    if (!$map->{$node})
                    {
                        $map->{$node} = 1;

                        $func->($func, $full_path);
                    }
                }
            }
        }

        closedir($dir);

        $depth--;
    };
}

sub process_file
{
    my $file_path = shift;

    if (file_is_provisioning_profile(get_file_name($file_path)))
	{
		find_app_ident_in_file($file_path);
	}
}

sub get_file_name
{
    my $file_path = shift;

    my $file = "";

    if ($file_path =~ m/([^\/\\]*)$/)
    {
        $file = $1;
    }

    return $file;
}

sub file_is_provisioning_profile
{
	return shift =~ m/\.(mobileprovision|provisionprofile)$/;
}

sub find_app_ident_in_file
{
	my $path = shift;

	my $file = get_file_name($path);
    my $ident = get_application_identifier(get_file_contents($path));
    my $date = get_expiration_date(get_file_contents($path));

    if (length $file and length $ident and length $date)
	{
		print "$path\t$ident\t$date\n";
	}
}

sub get_file_contents
{
	my $file = shift;

	my $contents = "";

	open(my $fh, "<", $file) || die "Could not open file [$file]: $!";

	while (<$fh>)
	{
		$contents .= $_;
	}

	close($fh);

	return $contents;
}

sub get_application_identifier
{
    my $content = shift;

	my $ident = "";

	if ($content =~ m/<key>application-identifier<\/key>\s*\n\s*<string>(.*)<\/string>/m)
	{
		$ident = $1;
	}

    return $ident;
}

sub get_expiration_date
{
    my $content = shift;

    my $ident = "";

    if ($content =~ m/<key>ExpirationDate<\/key>\s*\n\s*<date>(.*)<\/date>/m)
    {
        $ident = $1;
    }
    
    return $ident;
}
