#!/bin/bash

# Century tool to check .ipa files for validity

set -e

PLIST=/usr/libexec/PlistBuddy

CLEANUP=1
RUNALL=0
EXITCODE=0

if [ "$1" == "-d" ]
then
	shift
	CLEANUP=0
fi

if [ "$1" == "-a" ]
then
	shift
	RUNALL=1
fi

function cleanup
{
	popd > /dev/null 2>&1
	[ "$CLEANUP" == "1" ] && rm -rf "$TMPDIR"
}

function failure
{
	echo $1
	EXITCODE=$2
	shift
	shift
	echo $*
	if [ "$RUNALL" == "0" ]
	then
		cleanup
		exit $EXITCODE
	fi
}

function modified
{
	failure "The signature does not match the app contents (Modified after signing?)" 2
}

IPAFILE=$1
pushd $(dirname "$1") > /dev/null 2>&1
IPAFILE=$(pwd)/$(basename "$1")
popd > /dev/null 2>&1

TMPDIR=`mktemp -d apppackXXX`
pushd "$TMPDIR" > /dev/null 2>&1

unzip "$IPAFILE" > /dev/null 2>&1
APPFILE=$(ls -1d Payload/*)

codesign --entitlements :entitlements.xml -dvvvv "$APPFILE" > /dev/null 2>&1
ENT_APPID=$($PLIST -c "print :application-identifier" entitlements.xml)
ENT_APPID_NOPREFIX=$(echo $ENT_APPID | cut -d '.' -f 2-)
ENT_PREFIX=$(echo $ENT_APPID | cut -d '.' -f 1)
ENT_KEYCHAIN=$($PLIST -c "print :keychain-access-groups:0" entitlements.xml)

if [ "$ENT_APPID" != "$ENT_KEYCHAIN" ]
then
	failure "In the entitlements, the application ID does not match the keychain access group" 3 APPID=$ENT_APPID KEYCHAIN=$ENT_KEYCHAIN
fi

INFO_APPID=$($PLIST -c "print :CFBundleIdentifier" "$APPFILE/Info.plist")

if [ "$ENT_APPID_NOPREFIX" != "$INFO_APPID" ]
then
	failure "The Info.plist application ID does not match the entitlement application ID" 4 ENT=$ENT_APPID_NOPREFIX INFO=$INFO_APPID
fi

security cms -D -i "$APPFILE/embedded.mobileprovision" 2> /dev/null > embedded.mobileprovision

PROV_APPID=$($PLIST -c "print :Entitlements:application-identifier" embedded.mobileprovision)
PROV_APPID_NOPREFIX=$(echo $PROV_APPID | cut -d '.' -f 2-)
PROV_PREFIX=$(echo $PROV_APPID | cut -d '.' -f 1)
PROV_KEYCHAIN=$($PLIST -c "print :Entitlements:keychain-access-groups:0" embedded.mobileprovision)

if [ "$PROV_PREFIX" != "$ENT_PREFIX" ]
then
	failure "The embedded.mobileprovision application ID does not match the entitlement application ID" 5 PROV=$PROV_PREFIX ENT=$ENT_PREFIX
fi

if [ "$PROV_APPID" != "$PROV_PREFIX.*" ]
then
	if [ "$PROV_APPID_NOPREFIX" != "$INFO_APPID" ]
	then
		failure "Provisioning application ID does not match the Info.plist app ID" 9 PROV=$PROV_PREFIX INFO=$INFO_APPID
	fi
fi

# Check provision expiration date

PROV_EXPIRATION=$($PLIST -c "print :ExpirationDate" embedded.mobileprovision)
PROV_EXPIRATION_SECONDS=$(date -jf "%a %b %d %T %Z %Y" "$PROV_EXPIRATION" "+%s")
NOW_SECONDS=$(date "+%s")

[ "$NOW_SECONDS" -gt "$PROV_EXPIRATION_SECONDS" ] && failure "The embedded.mobileprovision has expired" 6 "$PROV_EXPIRATION"

if [ "$(expr $NOW_SECONDS + 1209600)" -gt "$PROV_EXPIRATION_SECONDS" -a "$NOW_SECONDS" -lt "$PROV_EXPIRATION_SECONDS" ]
then
	echo "WARNING: embedded.mobileprovision expires within the next two weeks!"
	echo "$PROV_EXPIRATION"
fi

# check for der entitlement
codesign -d --verbose=5 "$APPFILE/" 2>&1 | grep -E -q  -e "-7=[0-9a-f]{64}" || failure "codesign version incorrect, no der entitlement. See https://developer.apple.com/documentation/xcode/using-the-latest-code-signature-format"
codesign -d --verbose=5 "$APPFILE/" 2>&1 | grep -E -q  -e "-7=[0]{64}" && failure "codesign version incorrect, no der entitlement. See https://developer.apple.com/documentation/xcode/using-the-latest-code-signature-format"

# Extract signing keys
codesign --extract-certificates=cert -dvvvv "$APPFILE/" > /dev/null 2>&1

# Check key expiration
KEY_BEFORE=$(openssl x509 -text -inform der -in cert0 | grep "Not Before" | cut -d ':' -f 2-)
KEY_AFTER=$(openssl x509 -text -inform der -in cert0 | grep "Not After" | cut -d ':' -f 2-)
KEY_BEFORE_SECONDS=$(date -jf " %b %d %T %Y %Z" "$KEY_BEFORE" "+%s")
KEY_AFTER_SECONDS=$(date -jf " %b %d %T %Y %Z" "$KEY_AFTER" "+%s")

[ "$NOW_SECONDS" -lt "$KEY_BEFORE_SECONDS" ] && failure "Signing key is not yet valid" 7 "Not Before $KEY_BEFORE"

[ "$NOW_SECONDS" -gt "$KEY_AFTER_SECONDS" ] && failure "Signing key has expired" 8 "Not After $KEY_AFTER"

# Check code signatures
openssl x509 -text -inform der -in cert0 | grep -A 100 BEGIN  | grep -v -- ---- | tr -d '\n' > signing.cert
cat embedded.mobileprovision | grep -E -m 1 -o "<data>(.*)</data>" | sed -e 's,.*<data>\([^<]*\)</data>.*,\1,g' | tr -d '\n'> provision.cert

diff *.cert > /dev/null 2>&1 || failure "The embedded.mobileprovision signing key does not match the key used to sign the app" 1

codesign --no-strict --verify -vvvv -R='anchor apple generic and certificate 1[field.1.2.840.113635.100.6.2.1] exists and (certificate leaf[field.1.2.840.113635.100.6.1.2] exists or certificate leaf[field.1.2.840.113635.100.6.1.4] exists)' "$APPFILE" > /dev/null 2>&1 || modified

cleanup

[ "$EXITCODE" == "0" ] && echo "Looks good!"
exit $EXITCODE
